/*
 * Copyright (C) Eicon Technology Corporation, 2000.
 *
 * Eicon File Revision :    1.0  
 *
 * This software may be used and distributed according to the terms
 * of the GNU General Public License, incorporated herein by reference.
 *
 */

#ifndef DSPDIDS_H_
#define DSPDIDS_H_


/*---------------------------------------------------------------------------*/

#define DSP_DID_INVALID			0
#define DSP_DID_DIVA			1
#define DSP_DID_DIVA_PRO		2
#define DSP_DID_DIVA_PRO_20		3
#define DSP_DID_DIVA_PRO_PCCARD		4
#define DSP_DID_DIVA_SERVER_BRI_1M	5
#define DSP_DID_DIVA_SERVER_BRI_2M	6
#define DSP_DID_DIVA_SERVER_PRI_2M_TX	7
#define DSP_DID_DIVA_SERVER_PRI_2M_RX	8
#define DSP_DID_DIVA_SERVER_PRI_30M	9
#define DSP_DID_TASK_HSCX		100
#define DSP_DID_TASK_HSCX_PRI_2M_TX	101
#define DSP_DID_TASK_HSCX_PRI_2M_RX	102
#define DSP_DID_TASK_V110KRNL		200
#define DSP_DID_OVERLAY_V1100		201
#define DSP_DID_OVERLAY_V1101		202
#define DSP_DID_OVERLAY_V1102		203
#define DSP_DID_OVERLAY_V1103		204
#define DSP_DID_OVERLAY_V1104		205
#define DSP_DID_OVERLAY_V1105		206
#define DSP_DID_OVERLAY_V1106		207
#define DSP_DID_OVERLAY_V1107		208
#define DSP_DID_OVERLAY_V1108		209
#define DSP_DID_OVERLAY_V1109		210
#define DSP_DID_TASK_V110_PRI_2M_TX	220
#define DSP_DID_TASK_V110_PRI_2M_RX	221
#define DSP_DID_TASK_MODEM		300
#define DSP_DID_TASK_FAX05		400
#define DSP_DID_TASK_VOICE		500
#define DSP_DID_TASK_TIKRNL81		600
#define DSP_DID_OVERLAY_DIAL		601
#define DSP_DID_OVERLAY_V22		602
#define DSP_DID_OVERLAY_V32		603
#define DSP_DID_OVERLAY_FSK		604
#define DSP_DID_OVERLAY_FAX		605
#define DSP_DID_OVERLAY_VXX		606
#define DSP_DID_OVERLAY_V8		607
#define DSP_DID_OVERLAY_INFO		608
#define DSP_DID_OVERLAY_V34		609
#define DSP_DID_OVERLAY_DFX		610
#define DSP_DID_PARTIAL_OVERLAY_DIAL	611
#define DSP_DID_PARTIAL_OVERLAY_FSK	612
#define DSP_DID_PARTIAL_OVERLAY_FAX	613
#define DSP_DID_TASK_TIKRNL05		700


/*---------------------------------------------------------------------------*/

#endif

/*---------------------------------------------------------------------------*/
