
/*
 *
 * Copyright (C) Eicon Technology Corporation, 2000.
 *
 * This source file is supplied for the exclusive use with Eicon
 * Technology Corporation's range of DIVA Server Adapters.
 *
 * Eicon File Revision :    1.0  
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY OF ANY KIND WHATSOEVER INCLUDING ANY 
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


static char * cau_q931[] = {
  "0",
  "Unallocated (unassigned) number",
  "No route to specified transit network",
  "No route to destination",
  "4",
  "5",
  "Channel unacceptable",
  "Call awarded and being deliverd in an established channel",
  "8",
  "9",
  "10",
  "11",
  "12",
  "13",
  "14",
  "15",
  "Normal call clearing",
  "User busy",
  "No user responding",
  "No answer from user (user alerted)",
  "20",
  "Call rejected",
  "Number changed",
  "23",
  "24",
  "25",
  "Non-selected user clearing",
  "Destination out of order",
  "Invalid number format",
  "Facility rejected",
  "Response to STATUS ENQUIRY",
  "Normal, unspecified",
  "32",
  "33",
  "No circuit/channel available",
  "35",
  "36",
  "37",
  "Network out of order",
  "39",
  "40",
  "Temporary failure",
  "Switching equipment congestion",
  "Access information discarded",
  "Requested circuit/channel not available",
  "45",
  "46",
  "Resources unavailable, unspecified",
  "48",
  "Quality of service unavailable",
  "Requested facility not subscribed",
  "51",
  "52",
  "53",
  "54",
  "55",
  "56",
  "Bearer capability not authorized",
  "Bearer capabliity not presently available",
  "59",
  "60",
  "61",
  "62",
  "Service or option not available, unspecified",
  "64",
  "Bearer capability not implemented",
  "Channel type not implemented",
  "67",
  "68",
  "Requested facility not implemented",
  "Only restricted digital information bearer capability is available",
  "71",
  "72",
  "73",
  "74",
  "75",
  "76",
  "77",
  "78",
  "Service or option not implemented, unspecified",
  "80",
  "Invalid call reference value",
  "Identified channel does not exist",
  "A suspended call exists, but this call identity does not",
  "Call identity in use",
  "No call suspended",
  "Call having the requested call identity has been cleared",
  "87",
  "Incompatible destination",
  "89",
  "90",
  "Invalid transit network selection",
  "92",
  "93",
  "94",
  "Invalid message, unspecified",
  "Mandatory information element is missing",
  "Message type non-existent or not implemented",
  "Message not compatible with call state or not implemented",
  "Information element non-existent or not implemented",
  "Invalid information element contents",
  "Message not compatible with call state",
  "Recovery on timer expiry",
  "103",
  "104",
  "105",
  "106",
  "107",
  "108",
  "109",
  "110",
  "Protocol error, unspecified",
  "112",
  "113",
  "114",
  "115",
  "116",
  "117",
  "118",
  "119",
  "120",
  "121",
  "122",
  "123",
  "124",
  "125",
  "126",
  "Internetworking, unspecified"
};
